@@ -1,25 +1,27 @@
 package App::Prove;
 
 use strict;
+use vars qw($VERSION @ISA);
+
+use TAP::Object ();
 use TAP::Harness;
-use File::Find;
+use TAP::Parser::Utils qw( split_shell );
 use File::Spec;
 use Getopt::Long;
+use App::Prove::State;
 use Carp;
 
-use vars qw($VERSION);
-
 =head1 NAME
 
 App::Prove - Implements the C<prove> command.
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =cut
 
-$VERSION = '3.00';
+$VERSION = '3.17_01';
 
 =head1 DESCRIPTION
 
@@ -37,28 +39,29 @@ wrapper around an instance of this module.
 
 =cut
 
-my $IS_WIN32 = ( $^O =~ /^(MS)?Win32$/ );
-my $NEED_GLOB = $IS_WIN32;
+use constant IS_WIN32 => ( $^O =~ /^(MS)?Win32$/ );
+use constant IS_VMS => $^O eq 'VMS';
+use constant IS_UNIXY => !( IS_VMS || IS_WIN32 );
+
+use constant STATE_FILE => IS_UNIXY ? '.prove'   : '_prove';
+use constant RC_FILE    => IS_UNIXY ? '.proverc' : '_proverc';
 
 use constant PLUGINS => 'App::Prove::Plugin';
 
 my @ATTR;
 
 BEGIN {
+    @ISA = qw(TAP::Object);
+
     @ATTR = qw(
-      archive argv blib color directives exec failures fork formatter
-      harness includes modules plugins jobs lib merge parse quiet really_quiet recurse
-      backwards shuffle taint_fail taint_warn timer verbose
-      warnings_fail warnings_warn show_help show_man show_version
+      archive argv blib show_count color directives exec failures comments
+      formatter harness includes modules plugins jobs lib merge parse quiet
+      really_quiet recurse backwards shuffle taint_fail taint_warn timer
+      verbose warnings_fail warnings_warn show_help show_man show_version
+      state_class test_args state dry extension ignore_exit rules state_manager
+      normalize sources
     );
-    for my $attr (@ATTR) {
-        no strict 'refs';
-        *$attr = sub {
-            my $self = shift;
-            croak "$attr is read-only" if @_;
-            $self->{$attr};
-        };
-    }
+    __PACKAGE__->mk_methods(@ATTR);
 }
 
 =head1 METHODS
@@ -72,16 +75,19 @@ initializers may be passed.
 
 =cut
 
-sub new {
-    my $class = shift;
+# new() implementation supplied by TAP::Object
+
+sub _initialize {
+    my $self = shift;
     my $args = shift || {};
 
-    my $self = bless {
-        argv     => [],
-        includes => [],
-        modules  => [],
-        plugins  => [],
-    }, $class;
+    # setup defaults:
+    for my $key (
+        qw( argv rc_opts includes modules state plugins rules sources ))
+    {
+        $self->{$key} = [];
+    }
+    $self->{harness_class} = 'TAP::Harness';
 
     for my $attr (@ATTR) {
         if ( exists $args->{$attr} ) {
@@ -90,12 +96,55 @@ sub new {
             $self->{$attr} = $args->{$attr};
         }
     }
+
+    my %env_provides_default = (
+        HARNESS_TIMER => 'timer',
+    );
+
+    while ( my ( $env, $attr ) = each %env_provides_default ) {
+        $self->{$attr} = 1 if $ENV{$env};
+    }
+    $self->state_class('App::Prove::State');
     return $self;
 }
 
+=head3 C<state_class>
+
+Getter/setter for the name of the class used for maintaining state.  This
+class should either subclass from C<App::Prove::State> or provide an identical
+interface.
+
+=head3 C<state_manager>
+
+Getter/setter for the instance of the C<state_class>.
+
+=cut
+
+=head3 C<add_rc_file>
+
+    $prove->add_rc_file('myproj/.proverc');
+
+Called before C<process_args> to prepend the contents of an rc file to
+the options.
+
+=cut
+
+sub add_rc_file {
+    my ( $self, $rc_file ) = @_;
+
+    local *RC;
+    open RC, "<$rc_file" or croak "Can't read $rc_file ($!)";
+    while ( defined( my $line = <RC> ) ) {
+        push @{ $self->{rc_opts} },
+          grep { defined and not /^#/ }
+          $line =~ m{ ' ([^']*) ' | " ([^"]*) " | (\#.*) | (\S+) }xg;
+    }
+    close RC;
+}
+
 =head3 C<process_args>
 
-  $prove->process_args(@args);
+    $prove->process_args(@args);
 
 Processes the command-line arguments. Attributes will be set
 appropriately. Any filenames may be found in the C<argv> attribute.
@@ -105,34 +154,73 @@ Dies on invalid arguments.
 =cut
 
 sub process_args {
-    my ( $self, @args ) = @_;
+    my $self = shift;
+
+    my @rc = RC_FILE;
+    unshift @rc, glob '~/' . RC_FILE if IS_UNIXY;
+
+    # Preprocess meta-args.
+    my @args;
+    while ( defined( my $arg = shift ) ) {
+        if ( $arg eq '--norc' ) {
+            @rc = ();
+        }
+        elsif ( $arg eq '--rc' ) {
+            defined( my $rc = shift )
+              or croak "Missing argument to --rc";
+            push @rc, $rc;
+        }
+        elsif ( $arg =~ m{^--rc=(.+)$} ) {
+            push @rc, $1;
+        }
+        else {
+            push @args, $arg;
+        }
+    }
+
+    # Everything after the arisdottle '::' gets passed as args to
+    # test programs.
+    if ( defined( my $stop_at = _first_pos( '::', @args ) ) ) {
+        my @test_args = splice @args, $stop_at;
+        shift @test_args;
+        $self->{test_args} = \@test_args;
+    }
+
+    # Grab options from RC files
+    $self->add_rc_file($_) for grep -f, @rc;
+    unshift @args, @{ $self->{rc_opts} };
 
     if ( my @bad = map {"-$_"} grep {/^-(man|help)$/} @args ) {
         die "Long options should be written with two dashes: ",
           join( ', ', @bad ), "\n";
     }
 
-    # Allow cuddling the paths with the -I
-    @args = map { /^(-I)(.+)/ ? ( $1, $2 ) : $_ } @args;
+    # And finally...
 
     {
         local @ARGV = @args;
-        Getopt::Long::Configure( 'no_ignore_case', 'bundling' );
+        Getopt::Long::Configure(qw(no_ignore_case bundling pass_through));
 
         # Don't add coderefs to GetOptions
         GetOptions(
             'v|verbose'   => \$self->{verbose},
             'f|failures'  => \$self->{failures},
+            'o|comments'  => \$self->{comments},
             'l|lib'       => \$self->{lib},
             'b|blib'      => \$self->{blib},
             's|shuffle'   => \$self->{shuffle},
             'color!'      => \$self->{color},
+            'colour!'     => \$self->{color},
+            'count!'      => \$self->{show_count},
             'c'           => \$self->{color},
+            'D|dry'       => \$self->{dry},
+            'ext=s'       => \$self->{extension},
             'harness=s'   => \$self->{harness},
+            'ignore-exit' => \$self->{ignore_exit},
+            'source=s@'   => $self->{sources},
             'formatter=s' => \$self->{formatter},
             'r|recurse'   => \$self->{recurse},
             'reverse'     => \$self->{backwards},
-            'fork'        => \$self->{fork},
             'p|parse'     => \$self->{parse},
             'q|quiet'     => \$self->{quiet},
             'Q|QUIET'     => \$self->{really_quiet},
@@ -141,6 +229,7 @@ sub process_args {
             'I=s@'        => $self->{includes},
             'M=s@'        => $self->{modules},
             'P=s@'        => $self->{plugins},
+            'state=s@'    => $self->{state},
             'directives'  => \$self->{directives},
             'h|help|?'    => \$self->{show_help},
             'H|man'       => \$self->{show_man},
@@ -152,6 +241,8 @@ sub process_args {
             't'           => \$self->{taint_warn},
             'W'           => \$self->{warnings_fail},
             'w'           => \$self->{warnings_warn},
+            'normalize'   => \$self->{normalize},
+            'rules=s@'    => $self->{rules},
         ) or croak('Unable to continue');
 
         # Stash the remainder of argv for later
@@ -161,7 +252,13 @@ sub process_args {
     return;
 }
 
-sub _exit { exit( $_[1] || 0 ) }
+sub _first_pos {
+    my $want = shift;
+    for ( 0 .. $#_ ) {
+        return $_ if $_[$_] eq $want;
+    }
+    return;
+}
 
 sub _help {
     my ( $self, $verbosity ) = @_;
@@ -181,19 +278,23 @@ sub _help {
 sub _color_default {
     my $self = shift;
 
-    return -t STDOUT && !$IS_WIN32;
+    return -t STDOUT && !$ENV{HARNESS_NOTTY} && !IS_WIN32;
 }
 
 sub _get_args {
     my $self = shift;
 
-    $self->{harness_class} = 'TAP::Harness';
-
     my %args;
 
     if ( defined $self->color ? $self->color : $self->_color_default ) {
         $args{color} = 1;
     }
+    if ( !defined $self->show_count ) {
+        $args{show_count} = 1;
+    }
+    else {
+        $args{show_count} = $self->show_count;
+    }
 
     if ( $self->archive ) {
         $self->require_harness( archive => 'TAP::Harness::Archive' );
@@ -204,10 +305,6 @@ sub _get_args {
         $args{jobs} = $jobs;
     }
 
-    if ( my $fork = $self->fork ) {
-        $args{fork} = $fork;
-    }
-
     if ( my $harness_opt = $self->harness ) {
         $self->require_harness( harness => $harness_opt );
     }
@@ -216,6 +313,15 @@ sub _get_args {
         $args{formatter_class} = $formatter;
     }
 
+    foreach my $handler ( @{ $self->sources } ) {
+        my ( $name, $config ) = $self->_parse_source($handler);
+        $args{sources}->{$name} = $config;
+    }
+
+    if ( $self->ignore_exit ) {
+        $args{ignore_exit} = 1;
+    }
+
     if ( $self->taint_fail && $self->taint_warn ) {
         die '-t and -T are mutually exclusive';
     }
@@ -241,7 +347,7 @@ sub _get_args {
 
     $args{verbosity} = shift @verb_adj || 0;
 
-    for my $a (qw( merge failures timer directives )) {
+    for my $a (qw( merge failures comments timer directives normalize )) {
         $args{$a} = 1 if $self->$a();
     }
 
@@ -251,6 +357,23 @@ sub _get_args {
     $args{exec} = [ split( /\s+/, $self->exec ) ]
       if ( defined( $self->exec ) );
 
+    if ( defined( my $test_args = $self->test_args ) ) {
+        $args{test_args} = $test_args;
+    }
+
+    if ( @{ $self->rules } ) {
+        my @rules;
+        for ( @{ $self->rules } ) {
+            if (/^par=(.*)/) {
+                push @rules, $1;
+            }
+            elsif (/^seq=(.*)/) {
+                push @rules, { seq => $1 };
+            }
+        }
+        $args{rules} = { par => [@rules] };
+    }
+
     return ( \%args, $self->{harness_class} );
 }
 
@@ -262,7 +385,6 @@ sub _find_module {
 
     for my $pfx (@search) {
         my $name = join( '::', $pfx, $class );
-        print "$name\n";
         eval "require $name";
         return $name unless $@;
     }
@@ -273,19 +395,22 @@ sub _find_module {
 }
 
 sub _load_extension {
-    my ( $self, $class, @search ) = @_;
+    my ( $self, $name, @search ) = @_;
 
     my @args = ();
-    if ( $class =~ /^(.*?)=(.*)/ ) {
-        $class = $1;
+    if ( $name =~ /^(.*?)=(.*)/ ) {
+        $name = $1;
         @args = split( /,/, $2 );
     }
 
-    if ( my $name = $self->_find_module( $class, @search ) ) {
-        $name->import(@args);
+    if ( my $class = $self->_find_module( $name, @search ) ) {
+        $class->import(@args);
+        if ( $class->can('load') ) {
+            $class->load( { app_prove => $self, args => [@args] } );
+        }
     }
     else {
-        croak "Can't load module $class";
+        croak "Can't load module $name";
     }
 }
 
@@ -294,6 +419,30 @@ sub _load_extensions {
     $self->_load_extension( $_, @search ) for @$ext;
 }
 
+sub _parse_source {
+    my ( $self, $handler ) = @_;
+
+    # Load any options.
+    ( my $opt_name = lc $handler ) =~ s/::/-/g;
+    local @ARGV = @{ $self->{argv} };
+    my %config;
+    Getopt::Long::GetOptions(
+        "$opt_name-option=s%" => sub {
+            my ( undef, $k, $v ) = @_;
+            if ( exists $config{$k} ) {
+                $config{$k} = [ $config{$k} ]
+                  unless ref $config{$k} eq 'ARRAY';
+                push @{ $config{$k} } => $v;
+            }
+            else {
+                $config{$k} = $v;
+            }
+        }
+    );
+    $self->{argv} = \@ARGV;
+    return ( $handler, \%config );
+}
+
 =head3 C<run>
 
 Perform whatever actions the command line args specified. The C<prove>
@@ -303,13 +452,18 @@ command line tool consists of the following code:
 
     my $app = App::Prove->new;
     $app->process_args(@ARGV);
-    $app->run;
+    exit( $app->run ? 0 : 1 );  # if you need the exit code
 
 =cut
 
 sub run {
     my $self = shift;
 
+    unless ( $self->state_manager ) {
+        $self->state_manager(
+            $self->state_class->new( { store => STATE_FILE } ) );
+    }
+
     if ( $self->show_help ) {
         $self->_help(1);
     }
@@ -319,30 +473,61 @@ sub run {
     elsif ( $self->show_version ) {
         $self->print_version;
     }
+    elsif ( $self->dry ) {
+        print "$_\n" for $self->_get_tests;
+    }
     else {
 
         $self->_load_extensions( $self->modules );
         $self->_load_extensions( $self->plugins, PLUGINS );
 
-        my @tests = $self->_get_tests( @{ $self->argv } );
+        local $ENV{TEST_VERBOSE} = 1 if $self->verbose;
 
-        $self->_shuffle(@tests) if $self->shuffle;
-        @tests = reverse @tests if $self->backwards;
-
-        $self->_runtests( $self->_get_args, @tests );
+        return $self->_runtests( $self->_get_args, $self->_get_tests );
     }
 
-    return;
+    return 1;
+}
+
+sub _get_tests {
+    my $self = shift;
+
+    my $state = $self->state_manager;
+    my $ext   = $self->extension;
+    $state->extension($ext) if defined $ext;
+    if ( defined( my $state_switch = $self->state ) ) {
+        $state->apply_switch(@$state_switch);
+    }
+
+    my @tests = $state->get_tests( $self->recurse, @{ $self->argv } );
+
+    $self->_shuffle(@tests) if $self->shuffle;
+    @tests = reverse @tests if $self->backwards;
+
+    return @tests;
 }
 
 sub _runtests {
     my ( $self, $args, $harness_class, @tests ) = @_;
-    my $harness    = $harness_class->new($args);
+    my $harness = $harness_class->new($args);
+
+    my $state = $self->state_manager;
+
+    $harness->callback(
+        after_test => sub {
+            $state->observe_test(@_);
+        }
+    );
+
+    $harness->callback(
+        after_runtests => sub {
+            $state->commit(@_);
+        }
+    );
+
     my $aggregator = $harness->runtests(@tests);
 
-    $self->_exit( $aggregator->has_problems ? 1 : 0 );
-
-    return;
+    return !$aggregator->has_errors;
 }
 
 sub _get_switches {
@@ -363,6 +548,8 @@ sub _get_switches {
         push @switches, '-w';
     }
 
+    push @switches, split_shell( $ENV{HARNESS_PERL_SWITCHES} );
+
     return @switches ? \@switches : ();
 }
 
@@ -373,7 +560,7 @@ sub _get_lib {
         push @libs, 'lib';
     }
     if ( $self->blib ) {
-        push @libs, 'blib/lib';
+        push @libs, 'blib/lib', 'blib/arch';
     }
     if ( @{ $self->includes } ) {
         push @libs, @{ $self->includes };
@@ -386,63 +573,6 @@ sub _get_lib {
     return @libs ? \@libs : ();
 }
 
-sub _get_tests {
-    my $self = shift;
-    my @argv = @_;
-    my ( @tests, %tests );
-
-    unless (@argv) {
-        croak "No tests named and 't' directory not found"
-          unless -d 't';
-        @argv = 't';
-    }
-
-    # Do globbing on Win32.
-    if ($NEED_GLOB) {
-        @argv = map { glob "$_" } @argv;
-    }
-
-    foreach my $arg (@argv) {
-        if ( '-' eq $arg ) {
-            push @argv => <STDIN>;
-            chomp(@argv);
-            next;
-        }
-
-        if ( -d $arg ) {
-            my @files = $self->_expand_dir($arg);
-            foreach my $file (@files) {
-                push @tests => $file unless exists $tests{$file};
-            }
-            @tests{@files} = (1) x @files;
-        }
-        else {
-            push @tests => $arg unless exists $tests{$arg};
-            $tests{$arg} = 1;
-        }
-    }
-    return @tests;
-}
-
-sub _expand_dir {
-    my $self = shift;
-    my $dir  = shift;
-    my @tests;
-    if ( $self->recurse ) {
-        find(
-            {   follow => 1,    #21938
-                wanted =>
-                  sub { -f && /\.t$/ && push @tests => $File::Find::name }
-            },
-            $dir
-        );
-    }
-    else {
-        @tests = glob( File::Spec->catfile( $dir, '*.t' ) );
-    }
-    return sort @tests;
-}
-
 sub _shuffle {
     my $self = shift;
 
@@ -466,10 +596,15 @@ Load a harness replacement class.
 sub require_harness {
     my ( $self, $for, $class ) = @_;
 
-    eval("require $class");
-    die "$class is required to use the --$for feature: $@" if $@;
+    my ($class_name) = $class =~ /^(\w+(?:::\w+)*)/;
 
-    $self->{harness_class} = $class;
+    # Emulate Perl's -MModule=arg1,arg2 behaviour
+    $class =~ s!^(\w+(?:::\w+)*)=(.*)$!$1 split(/,/,q{$2})!;
+
+    eval("use $class;");
+    die "$class_name is required to use the --$for feature: $@" if $@;
+
+    $self->{harness_class} = $class_name;
 
     return;
 }
@@ -517,16 +652,22 @@ calling C<run>.
 
 =item C<directives>
 
+=item C<dry>
+
 =item C<exec>
 
+=item C<extension>
+
 =item C<failures>
 
-=item C<fork>
+=item C<comments>
 
 =item C<formatter>
 
 =item C<harness>
 
+=item C<ignore_exit>
+
 =item C<includes>
 
 =item C<jobs>
@@ -547,6 +688,10 @@ calling C<run>.
 
 =item C<recurse>
 
+=item C<rules>
+
+=item C<show_count>
+
 =item C<show_help>
 
 =item C<show_man>
@@ -555,10 +700,16 @@ calling C<run>.
 
 =item C<shuffle>
 
+=item C<state>
+
+=item C<state_class>
+
 =item C<taint_fail>
 
 =item C<taint_warn>
 
+=item C<test_args>
+
 =item C<timer>
 
 =item C<verbose>
@@ -568,3 +719,88 @@ calling C<run>.
 =item C<warnings_warn>
 
 =back
+
+=head1 PLUGINS
+
+C<App::Prove> provides support for 3rd-party plugins.  These are currently
+loaded at run-time, I<after> arguments have been parsed (so you can not
+change the way arguments are processed, sorry), typically with the
+C<< -PI<plugin> >> switch, eg:
+
+  prove -PMyPlugin
+
+This will search for a module named C<App::Prove::Plugin::MyPlugin>, or failing
+that, C<MyPlugin>.  If the plugin can't be found, C<prove> will complain & exit.
+
+You can pass an argument to your plugin by appending an C<=> after the plugin
+name, eg C<-PMyPlugin=foo>.  You can pass multiple arguments using commas:
+
+  prove -PMyPlugin=foo,bar,baz
+
+These are passed in to your plugin's C<load()> class method (if it has one),
+along with a reference to the C<App::Prove> object that is invoking your plugin:
+
+  sub load {
+      my ($class, $p) = @_;
+
+      my @args = @{ $p->{args} };
+      # @args will contain ( 'foo', 'bar', 'baz' )
+      $p->{app_prove}->do_something;
+      ...
+  }
+
+Note that the user's arguments are also passed to your plugin's C<import()>
+function as a list, eg:
+
+  sub import {
+      my ($class, @args) = @_;
+      # @args will contain ( 'foo', 'bar', 'baz' )
+      ...
+  }
+
+This is for backwards compatibility, and may be deprecated in the future.
+
+=head2 Sample Plugin
+
+Here's a sample plugin, for your reference:
+
+  package App::Prove::Plugin::Foo;
+
+  # Sample plugin, try running with:
+  # prove -PFoo=bar -r -j3
+  # prove -PFoo -Q
+  # prove -PFoo=bar,My::Formatter
+
+  use strict;
+  use warnings;
+
+  sub load {
+      my ($class, $p) = @_;
+      my @args = @{ $p->{args} };
+      my $app  = $p->{app_prove};
+
+      print "loading plugin: $class, args: ", join(', ', @args ), "\n";
+
+      # turn on verbosity
+      $app->verbose( 1 );
+
+      # set the formatter?
+      $app->formatter( $args[1] ) if @args > 1;
+
+      # print some of App::Prove's state:
+      for my $attr (qw( jobs quiet really_quiet recurse verbose )) {
+          my $val = $app->$attr;
+          $val    = 'undef' unless defined( $val );
+          print "$attr: $val\n";
+      }
+
+      return 1;
+  }
+
+  1;
+
+=head1 SEE ALSO
+
+L<prove>, L<TAP::Harness>
+
+=cut
