@@ -1,26 +1,29 @@
 package TAP::Base;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION @ISA);
+
+use TAP::Object;
+
+@ISA = qw(TAP::Object);
 
 =head1 NAME
 
-TAP::Base - Base class that provides common functionality to L<TAP::Parser> and L<TAP::Harness>
+TAP::Base - Base class that provides common functionality to L<TAP::Parser>
+and L<TAP::Harness>
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =cut
 
-$VERSION = '3.00';
+$VERSION = '3.17_01';
 
-my $GOT_TIME_HIRES;
-
-BEGIN {
+use constant GOT_TIME_HIRES => do {
     eval 'use Time::HiRes qw(time);';
-    $GOT_TIME_HIRES = $@ ? 0 : 1;
-}
+    $@ ? 0 : 1;
+};
 
 =head1 SYNOPSIS
 
@@ -47,17 +50,8 @@ C<TAP::Base> provides callback management.
 
 =head2 Class Methods
 
-=head3 C<new>
-
 =cut
 
-sub new {
-    my ( $class, $arg_for ) = @_;
-
-    my $self = bless {}, $class;
-    return $self->_initialize($arg_for);
-}
-
 sub _initialize {
     my ( $self, $arg_for, $ok_callback ) = @_;
 
@@ -65,8 +59,8 @@ sub _initialize {
 
     $self->{ok_callbacks} = \%ok_map;
 
-    if ( exists $arg_for->{callbacks} ) {
-        while ( my ( $event, $callback ) = each %{ $arg_for->{callbacks} } ) {
+    if ( my $cb = delete $arg_for->{callbacks} ) {
+        while ( my ( $event, $callback ) = each %$cb ) {
             $self->callback( $event, $callback );
         }
     }
@@ -116,14 +110,6 @@ sub _make_callback {
     return map { $_->(@_) } @$cb;
 }
 
-sub _croak {
-    my ( $self, $message ) = @_;
-    require Carp;
-    Carp::croak($message);
-
-    return;
-}
-
 =head3 C<get_time>
 
 Return the current time using Time::HiRes if available.
@@ -138,6 +124,6 @@ Return true if the time returned by get_time is high resolution (i.e. if Time::H
 
 =cut
 
-sub time_is_hires { return $GOT_TIME_HIRES }
+sub time_is_hires { return GOT_TIME_HIRES }
 
 1;
