@@ -2,7 +2,11 @@ package TAP::Parser::Aggregator;
 
 use strict;
 use Benchmark;
-use vars qw($VERSION);
+use vars qw($VERSION @ISA);
+
+use TAP::Object ();
+
+@ISA = qw(TAP::Object);
 
 =head1 NAME
 
@@ -10,11 +14,11 @@ TAP::Parser::Aggregator - Aggregate TAP::Parser results
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =cut
 
-$VERSION = '3.00';
+$VERSION = '3.17_01';
 
 =head1 SYNOPSIS
 
@@ -51,6 +55,8 @@ Returns a new C<TAP::Parser::Aggregator> object.
 
 =cut
 
+# new() implementation supplied by TAP::Object
+
 my %SUMMARY_METHOD_FOR;
 
 BEGIN {    # install summary methods
@@ -65,7 +71,8 @@ BEGIN {    # install summary methods
       wait
       exit
     );
-    $SUMMARY_METHOD_FOR{total} = 'tests_run';
+    $SUMMARY_METHOD_FOR{total}   = 'tests_run';
+    $SUMMARY_METHOD_FOR{planned} = 'tests_planned';
 
     foreach my $method ( keys %SUMMARY_METHOD_FOR ) {
         next if 'total' eq $method;
@@ -79,13 +86,6 @@ BEGIN {    # install summary methods
     }
 }    # end install summary methods
 
-sub new {
-    my ($class) = @_;
-    my $self = bless {}, $class;
-    $self->_initialize;
-    return $self;
-}
-
 sub _initialize {
     my ($self) = @_;
     $self->{parser_for}  = {};
@@ -117,12 +117,20 @@ The C<$parser> is a L<TAP::Parser|TAP::Parser> object.
 sub add {
     my ( $self, $description, $parser ) = @_;
     if ( exists $self->{parser_for}{$description} ) {
-        $self->_croak("You already have a parser for ($description)");
+        $self->_croak( "You already have a parser for ($description)."
+              . " Perhaps you have run the same test twice." );
     }
     push @{ $self->{parse_order} } => $description;
     $self->{parser_for}{$description} = $parser;
 
     while ( my ( $summary, $method ) = each %SUMMARY_METHOD_FOR ) {
+
+        # Slightly nasty. Instead we should maybe have 'cooked' accessors
+        # for results that may be masked by the parser.
+        next
+          if ( $method eq 'exit' || $method eq 'wait' )
+          && $parser->ignore_exit;
+
         if ( my $count = $parser->$method() ) {
             $self->{$summary} += $count;
             push @{ $self->{"descriptions_for_$summary"} } => $description;
@@ -177,7 +185,8 @@ sub _get_parsers {
 
 =head3 C<descriptions>
 
-Get an array of descriptions in the order in which they were added to the aggregator.
+Get an array of descriptions in the order in which they were added to
+the aggregator.
 
 =cut
 
@@ -248,7 +257,8 @@ Return true if all the tests passed and no parse errors were detected.
 
 sub all_passed {
     my $self = shift;
-    return $self->total
+    return
+         $self->total
       && $self->total == $self->passed
       && !$self->has_errors;
 }
@@ -264,13 +274,13 @@ Depending on the outcome of the tests returns 'PASS', 'FAIL' or
 sub get_status {
     my $self = shift;
 
-    if ( my $total = $self->total ) {
-        return $total == $self->passed
-          && !$self->has_errors ? 'PASS' : 'FAIL';
-    }
-    else {
-        return 'NOTESTS';
-    }
+    my $total  = $self->total;
+    my $passed = $self->passed;
+
+    return
+        ( $self->has_errors || $total != $passed ) ? 'FAIL'
+      : $total ? 'PASS'
+      :          'NOTESTS';
 }
 
 ##############################################################################
@@ -290,6 +300,8 @@ for an explanation of description.
 
 =item * passed
 
+=item * planned
+
 =item * skipped
 
 =item * todo
@@ -359,7 +371,7 @@ Returns true if I<any> of the parsers failed.  This includes:
 
 =item * Failed tests
 
-=item * Parse erros
+=item * Parse errors
 
 =item * Bad exit or wait status
 
@@ -369,7 +381,8 @@ Returns true if I<any> of the parsers failed.  This includes:
 
 sub has_errors {
     my $self = shift;
-    return $self->failed
+    return
+         $self->failed
       || $self->parse_errors
       || $self->exit
       || $self->wait;
@@ -392,12 +405,6 @@ sub todo_failed {
     goto &todo_passed;
 }
 
-sub _croak {
-    my $proto = shift;
-    require Carp;
-    Carp::croak(@_);
-}
-
 =head1 See Also
 
 L<TAP::Parser>
