@@ -1,52 +1,73 @@
 package TAP::Parser::Grammar;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION @ISA);
 
-use TAP::Parser::Result          ();
+use TAP::Object                  ();
+use TAP::Parser::ResultFactory   ();
 use TAP::Parser::YAMLish::Reader ();
 
+@ISA = qw(TAP::Object);
+
 =head1 NAME
 
 TAP::Parser::Grammar - A grammar for the Test Anything Protocol.
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =cut
 
-$VERSION = '3.00';
+$VERSION = '3.17_01';
+
+=head1 SYNOPSIS
+
+  use TAP::Parser::Grammar;
+  my $grammar = $self->make_grammar({
+    iterator => $tap_parser_iterator,
+    parser   => $tap_parser,
+    version  => 12,
+  });
+
+  my $result = $grammar->tokenize;
 
 =head1 DESCRIPTION
 
-C<TAP::Parser::Grammar> tokenizes lines from a TAP stream and constructs
-L<TAP::Parser::Result> subclasses to represent the tokens.
+C<TAP::Parser::Grammar> tokenizes lines from a L<TAP::Parser::Iterator> and
+constructs L<TAP::Parser::Result> subclasses to represent the tokens.
 
 Do not attempt to use this class directly.  It won't make sense.  It's mainly
 here to ensure that we will be able to have pluggable grammars when TAP is
 expanded at some future date (plus, this stuff was really cluttering the
 parser).
 
-=cut
-
-##############################################################################
+=head1 METHODS
 
 =head2 Class Methods
 
-
 =head3 C<new>
 
-  my $grammar = TAP::Grammar->new($stream);
+  my $grammar = TAP::Parser::Grammar->new({
+      iterator => $iterator,
+      parser   => $parser,
+      version  => $version,
+  });
 
-Returns TAP grammar object that will parse the specified stream.
+Returns L<TAP::Parser> grammar object that will parse the TAP stream from the
+specified iterator.  Both C<iterator> and C<parser> are required arguments.
+If C<version> is not set it defaults to C<12> (see L</set_version> for more
+details).
 
 =cut
 
-sub new {
-    my ( $class, $stream ) = @_;
-    my $self = bless { stream => $stream }, $class;
-    $self->set_version(12);
+# new() implementation supplied by TAP::Object
+sub _initialize {
+    my ( $self, $args ) = @_;
+    $self->{iterator} = $args->{iterator};    # TODO: accessor
+    $self->{iterator} ||= $args->{stream};    # deprecated
+    $self->{parser} = $args->{parser};        # TODO: accessor
+    $self->set_version( $args->{version} || 12 );
     return $self;
 }
 
@@ -73,6 +94,7 @@ my %language_for;
                 my ( $self, $line ) = @_;
                 my ( $tests_planned, $tail ) = ( $1, $2 );
                 my $explanation = undef;
+                my $skip        = '';
 
                 if ( $tail =~ /^todo((?:\s+\d+)+)/ ) {
                     my @todo = split /\s+/, _trim($1);
@@ -81,19 +103,18 @@ my %language_for;
                         '',    \@todo
                     );
                 }
-                elsif ( $tail =~ /^#\s*SKIP\S*\s*(.*)\z/i ) {
-                    $explanation = $1;
+                elsif ( 0 == $tests_planned ) {
+                    $skip = 'SKIP';
+
+                    # If we can't match # SKIP the directive should be undef.
+                    ($explanation) = $tail =~ /^#\s*SKIP\S*\s+(.*)/i;
                 }
                 elsif ( $tail !~ /^\s*$/ ) {
                     return $self->_make_unknown_token($line);
                 }
 
-                my $skip
-                  = ( 0 == $tests_planned || defined $explanation )
-                  ? 'SKIP'
-                  : '';
-
                 $explanation = '' unless defined $explanation;
+
                 return $self->_make_plan_token(
                     $line, $tests_planned, $skip,
                     $explanation, []
@@ -114,7 +135,7 @@ my %language_for;
                     $line, $ok, $num,
                     $desc
                 );
-              }
+            },
         },
         test => {
             syntax  => qr/^($ok) \s* ($num)? \s* (.*) \z/x,
@@ -129,8 +150,8 @@ my %language_for;
                     ( $desc, $dir, $explanation ) = ( $1, $2, $3 );
                 }
                 return $self->_make_test_token(
-                    $line,   $ok, $num, $desc,
-                    uc $dir, $explanation
+                    $line, $ok, $num, $desc,
+                    $dir,  $explanation
                 );
             },
         },
@@ -181,6 +202,15 @@ my %language_for;
                 return $self->_make_yaml_token( $pad, $marker );
             },
         },
+        pragma => {
+            syntax =>
+              qr/^ pragma \s+ ( [-+] \w+ \s* (?: , \s* [-+] \w+ \s* )* ) $/x,
+            handler => sub {
+                my ( $self, $line ) = @_;
+                my $pragmas = $1;
+                return $self->_make_pragma_token( $line, $pragmas );
+            },
+        },
     );
 
     %language_for = (
@@ -190,7 +220,7 @@ my %language_for;
         '13' => {
             tokens => \%v13,
             setup  => sub {
-                shift->{stream}->handle_unicode;
+                shift->{iterator}->handle_unicode;
             },
         },
     );
@@ -215,7 +245,8 @@ sub set_version {
     my $version = shift;
 
     if ( my $language = $language_for{$version} ) {
-        $self->{tokens} = $language->{tokens};
+        $self->{version} = $version;
+        $self->{tokens}  = $language->{tokens};
 
         if ( my $setup = $language->{setup} ) {
             $self->$setup();
@@ -255,9 +286,11 @@ current line of TAP.
 sub tokenize {
     my $self = shift;
 
-    my $stream = $self->{stream};
-    my $line   = $stream->next;
-    return unless defined $line;
+    my $line = $self->{iterator}->next;
+    unless ( defined $line ) {
+        delete $self->{parser};    # break circular ref
+        return;
+    }
 
     my $token;
 
@@ -271,7 +304,7 @@ sub tokenize {
 
     $token = $self->_make_unknown_token($line) unless $token;
 
-    return TAP::Parser::Result->new($token);
+    return $self->{parser}->make_result($token);
 }
 
 ##############################################################################
@@ -350,10 +383,14 @@ sub _make_version_token {
 sub _make_plan_token {
     my ( $self, $line, $tests_planned, $directive, $explanation, $todo ) = @_;
 
-    if ( $directive eq 'SKIP' && 0 != $tests_planned ) {
+    if (   $directive eq 'SKIP'
+        && 0 != $tests_planned
+        && $self->{version} < 13 )
+    {
         warn
           "Specified SKIP directive in plan but more than 0 tests ($line)\n";
     }
+
     return {
         type          => 'plan',
         raw           => $line,
@@ -366,16 +403,15 @@ sub _make_plan_token {
 
 sub _make_test_token {
     my ( $self, $line, $ok, $num, $desc, $dir, $explanation ) = @_;
-    my %test = (
+    return {
         ok          => $ok,
         test_num    => $num,
         description => _trim($desc),
-        directive   => uc($dir),
+        directive   => ( defined $dir ? uc $dir : '' ),
         explanation => _trim($explanation),
         raw         => $line,
         type        => 'test',
-    );
-    return \%test;
+    };
 }
 
 sub _make_unknown_token {
@@ -391,7 +427,7 @@ sub _make_comment_token {
     return {
         type    => 'comment',
         raw     => $line,
-        comment => _trim($1)
+        comment => _trim($comment)
     };
 }
 
@@ -409,7 +445,7 @@ sub _make_yaml_token {
 
     my $yaml = TAP::Parser::YAMLish::Reader->new;
 
-    my $stream = $self->{stream};
+    my $iterator = $self->{iterator};
 
     # Construct a reader that reads from our input stripping leading
     # spaces from each line.
@@ -418,7 +454,7 @@ sub _make_yaml_token {
     my @extra  = ($marker);
     my $reader = sub {
         return shift @extra if @extra;
-        my $line = $stream->next;
+        my $line = $iterator->next;
         return $2 if $line =~ $strip;
         return;
     };
@@ -437,13 +473,27 @@ sub _make_yaml_token {
     };
 }
 
+sub _make_pragma_token {
+    my ( $self, $line, $pragmas ) = @_;
+    return {
+        type    => 'pragma',
+        raw     => $line,
+        pragmas => [ split /\s*,\s*/, _trim($pragmas) ],
+    };
+}
+
 sub _trim {
-    my $data = shift || '';
+    my $data = shift;
+
+    return '' unless defined $data;
+
     $data =~ s/^\s+//;
     $data =~ s/\s+$//;
     return $data;
 }
 
+1;
+
 =head1 TAP GRAMMAR
 
 B<NOTE:>  This grammar is slightly out of date.  There's still some discussion
@@ -515,7 +565,18 @@ A formal grammar would look similar to the following:
  positiveInteger    ::= ( digit - '0' ) {digit}
  nonNegativeInteger ::= digit {digit}
 
+=head1 SUBCLASSING
+
+Please see L<TAP::Parser/SUBCLASSING> for a subclassing overview.
+
+If you I<really> want to subclass L<TAP::Parser>'s grammar the best thing to
+do is read through the code.  There's no easy way of summarizing it here.
+
+=head1 SEE ALSO
+
+L<TAP::Object>,
+L<TAP::Parser>,
+L<TAP::Parser::Iterator>,
+L<TAP::Parser::Result>,
 
 =cut
-
-1;
