@@ -1,56 +1,67 @@
 package TAP::Parser::Iterator::Process;
 
 use strict;
-
-use TAP::Parser::Iterator ();
-
 use vars qw($VERSION @ISA);
 
-@ISA = 'TAP::Parser::Iterator';
-
+use TAP::Parser::Iterator ();
 use Config;
 use IO::Handle;
 
+@ISA = 'TAP::Parser::Iterator';
+
 my $IS_WIN32 = ( $^O =~ /^(MS)?Win32$/ );
 
 =head1 NAME
 
-TAP::Parser::Iterator::Process - Internal TAP::Parser Iterator
+TAP::Parser::Iterator::Process - Iterator for proccess-based TAP sources
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =cut
 
-$VERSION = '3.00';
+$VERSION = '3.17_01';
 
 =head1 SYNOPSIS
 
-  use TAP::Parser::Iterator;
-  my $it = TAP::Parser::Iterator::Process->new(@args);
-
+  use TAP::Parser::Iterator::Process;
+  my %args = (
+   command  => ['python', 'setup.py', 'test'],
+   merge    => 1,
+   setup    => sub { ... },
+   teardown => sub { ... },
+  );
+  my $it   = TAP::Parser::Iterator::Process->new(\%args);
   my $line = $it->next;
 
-Originally ripped off from L<Test::Harness>.
-
 =head1 DESCRIPTION
 
-B<FOR INTERNAL USE ONLY!>
+This is a simple iterator wrapper for executing external processes, used by
+L<TAP::Parser>.  Unless you're writing a plugin or subclassing, you probably
+won't need to use this module directly.
 
-This is a simple iterator wrapper for processes.
+=head1 METHODS
 
 =head2 Class Methods
 
 =head3 C<new>
 
-Create an iterator.
+Create an iterator.  Expects one argument containing a hashref of the form:
+
+   command  => \@command_to_execute
+   merge    => $attempt_merge_stderr_and_stdout?
+   setup    => $callback_to_setup_command
+   teardown => $callback_to_teardown_command
+
+Tries to uses L<IPC::Open3> & L<IO::Select> to communicate with the spawned
+process if they are available.  Falls back onto C<open()>.
 
 =head2 Instance Methods
 
 =head3 C<next>
 
-Iterate through it, of course.
+Iterate through the process output, of course.
 
 =head3 C<next_raw>
 
@@ -84,13 +95,30 @@ sub _use_open3 {
     return 1;
 }
 
-sub new {
-    my $class = shift;
-    my $args  = shift;
+{
+    my $got_unicode;
+
+    sub _get_unicode {
+        return $got_unicode if defined $got_unicode;
+        eval 'use Encode qw(decode_utf8);';
+        $got_unicode = $@ ? 0 : 1;
+
+    }
+}
+
+# new() implementation supplied by TAP::Object
+
+sub _initialize {
+    my ( $self, $args ) = @_;
 
     my @command = @{ delete $args->{command} || [] }
       or die "Must supply a command to execute";
 
+    $self->{command} = [@command];
+
+    # Private. Used to frig with chunk size during testing.
+    my $chunk_size = delete $args->{_chunk_size} || 65536;
+
     my $merge = delete $args->{merge};
     my ( $pid, $err, $sel );
 
@@ -100,7 +128,7 @@ sub new {
 
     my $out = IO::Handle->new;
 
-    if ( $class->_use_open3 ) {
+    if ( $self->_use_open3 ) {
 
         # HOTPATCH {{{
         my $xclose = \&IPC::Open3::xclose;
@@ -144,13 +172,12 @@ sub new {
           or die "Could not execute ($command): $!";
     }
 
-    my $self = bless {
-        out  => $out,
-        err  => $err,
-        sel  => $sel,
-        pid  => $pid,
-        exit => undef,
-    }, $class;
+    $self->{out}        = $out;
+    $self->{err}        = $err;
+    $self->{sel}        = $sel;
+    $self->{pid}        = $pid;
+    $self->{exit}       = undef;
+    $self->{chunk_size} = $chunk_size;
 
     if ( my $teardown = delete $args->{teardown} ) {
         $self->{teardown} = sub {
@@ -169,11 +196,27 @@ Upgrade the input stream to handle UTF8.
 
 sub handle_unicode {
     my $self = shift;
-    if ( $] >= 5.008 ) {
-        my ( $out, $err ) = ( $self->{out}, $self->{err} );
-        eval 'binmode($out, ":utf8")';
-        eval 'binmode($err, ":utf8")' if ref $err;
+
+    if ( $self->{sel} ) {
+        if ( _get_unicode() ) {
+
+            # Make sure our iterator has been constructed and...
+            my $next = $self->{_next} ||= $self->_next;
+
+            # ...wrap it to do UTF8 casting
+            $self->{_next} = sub {
+                my $line = $next->();
+                return decode_utf8($line) if defined $line;
+                return;
+            };
+        }
     }
+    else {
+        if ( $] >= 5.008 ) {
+            eval 'binmode($self->{out}, ":utf8")';
+        }
+    }
+
 }
 
 ##############################################################################
@@ -181,53 +224,85 @@ sub handle_unicode {
 sub wait { shift->{wait} }
 sub exit { shift->{exit} }
 
-sub next_raw {
+sub _next {
     my $self = shift;
 
     if ( my $out = $self->{out} ) {
-
-        # If we have an IO::Select we need to poll it.
         if ( my $sel = $self->{sel} ) {
-            my $err  = $self->{err};
-            my $flip = 0;
+            my $err        = $self->{err};
+            my @buf        = ();
+            my $partial    = '';                    # Partial line
+            my $chunk_size = $self->{chunk_size};
+            return sub {
+                return shift @buf if @buf;
 
-            # Loops forever while we're reading from STDERR
-            while ( my @ready = $sel->can_read ) {
+                READ:
+                while ( my @ready = $sel->can_read ) {
+                    for my $fh (@ready) {
+                        my $got = sysread $fh, my ($chunk), $chunk_size;
 
-                # Load balancing :)
-                @ready = reverse @ready if $flip;
-                $flip = !$flip;
-
-                for my $fh (@ready) {
-                    if ( defined( my $line = <$fh> ) ) {
-                        if ( $fh == $err ) {
-                            warn $line;
+                        if ( $got == 0 ) {
+                            $sel->remove($fh);
+                        }
+                        elsif ( $fh == $err ) {
+                            print STDERR $chunk;    # echo STDERR
                         }
                         else {
-                            chomp $line;
-                            return $line;
+                            $chunk   = $partial . $chunk;
+                            $partial = '';
+
+                            # Make sure we have a complete line
+                            unless ( substr( $chunk, -1, 1 ) eq "\n" ) {
+                                my $nl = rindex $chunk, "\n";
+                                if ( $nl == -1 ) {
+                                    $partial = $chunk;
+                                    redo READ;
+                                }
+                                else {
+                                    $partial = substr( $chunk, $nl + 1 );
+                                    $chunk = substr( $chunk, 0, $nl );
+                                }
+                            }
+
+                            push @buf, split /\n/, $chunk;
+                            return shift @buf if @buf;
                         }
                     }
-                    else {
-                        $sel->remove($fh);
-                    }
                 }
-            }
+
+                # Return partial last line
+                if ( length $partial ) {
+                    my $last = $partial;
+                    $partial = '';
+                    return $last;
+                }
+
+                $self->_finish;
+                return;
+            };
         }
         else {
-
-            # Only one handle: just a simple read
-            if ( defined( my $line = <$out> ) ) {
-                chomp $line;
-                return $line;
-            }
+            return sub {
+                if ( defined( my $line = <$out> ) ) {
+                    chomp $line;
+                    return $line;
+                }
+                $self->_finish;
+                return;
+            };
         }
     }
+    else {
+        return sub {
+            $self->_finish;
+            return;
+        };
+    }
+}
 
-    # We only get here when the stream(s) is/are exhausted
-    $self->_finish;
-
-    return;
+sub next_raw {
+    my $self = shift;
+    return ( $self->{_next} ||= $self->_next )->();
 }
 
 sub _finish {
@@ -235,6 +310,10 @@ sub _finish {
 
     my $status = $?;
 
+    # Avoid circular refs
+    $self->{_next} = sub {return}
+      if $] >= 5.006;
+
     # If we have a subprocess we need to wait for it to terminate
     if ( defined $self->{pid} ) {
         if ( $self->{pid} == waitpid( $self->{pid}, 0 ) ) {
@@ -281,3 +360,16 @@ sub get_select_handles {
 }
 
 1;
+
+=head1 ATTRIBUTION
+
+Originally ripped off from L<Test::Harness>.
+
+=head1 SEE ALSO
+
+L<TAP::Object>,
+L<TAP::Parser>,
+L<TAP::Parser::Iterator>,
+
+=cut
+
