@@ -1,10 +1,12 @@
 package TAP::Parser::YAMLish::Reader;
 
 use strict;
+use vars qw($VERSION @ISA);
 
-use vars qw{$VERSION};
+use TAP::Object ();
 
-$VERSION = '3.00';
+@ISA     = 'TAP::Object';
+$VERSION = '3.17_01';
 
 # TODO:
 #   Handle blessed object syntax
@@ -17,16 +19,12 @@ my %UNESCAPES = (
 );
 
 my $QQ_STRING    = qr{ " (?:\\. | [^"])* " }x;
-my $HASH_LINE    = qr{ ^ ($QQ_STRING|\S+) \s* : (?: \s+ (.+?) \s* )? $ }x;
+my $HASH_LINE    = qr{ ^ ($QQ_STRING|\S+) \s* : \s* (?: (.+?) \s* )? $ }x;
 my $IS_HASH_KEY  = qr{ ^ [\w\'\"] }x;
 my $IS_END_YAML  = qr{ ^ \.\.\. \s* $ }x;
 my $IS_QQ_STRING = qr{ ^ $QQ_STRING $ }x;
 
-# Create an empty TAP::Parser::YAMLish::Reader object
-sub new {
-    my $class = shift;
-    bless {}, $class;
-}
+# new() implementation supplied by TAP::Object
 
 sub read {
     my $self = shift;
@@ -40,6 +38,7 @@ sub read {
 
     # Prime the reader
     $self->_next;
+    return unless $self->{next};
 
     my $doc = $self->_read;
 
@@ -58,15 +57,7 @@ sub read {
     return $doc;
 }
 
-sub get_raw {
-    my $self = shift;
-
-    if ( defined( my $capture = $self->{capture} ) ) {
-        return join( "\n", @$capture ) . "\n";
-    }
-
-    return '';
-}
+sub get_raw { join( "\n", grep defined, @{ shift->{capture} || [] } ) . "\n" }
 
 sub _peek {
     my $self = shift;
@@ -151,7 +142,9 @@ sub _read_scalar {
             $self->_next;
             my ( $next, $ind ) = $self->_peek;
             last if $ind < $indent;
-            push @multiline, $next;
+
+            my $pad = $string eq '|' ? ( ' ' x ( $ind - $indent ) ) : '';
+            push @multiline, $pad . $next;
         }
 
         return join( ( $string eq '>' ? ' ' : "\n" ), @multiline ) . "\n";
@@ -277,7 +270,7 @@ TAP::Parser::YAMLish::Reader - Read YAMLish data from iterator
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =head1 SYNOPSIS
 
@@ -301,7 +294,7 @@ C<TAP::Parser::YAMLish::Reader> object.
 
 =head3 C<read>
 
- my $got = $reader->read($stream);
+ my $got = $reader->read($iterator);
 
 Read YAMLish from a L<TAP::Parser::Iterator> and return the data structure it
 represents.
@@ -326,9 +319,9 @@ L<http://use.perl.org/~Alias/journal/29427>
 
 =head1 COPYRIGHT
 
-Copyright 2007 Andy Armstrong.
+Copyright 2007-2008 Andy Armstrong.
 
-Portions copyright 2006-2007 Adam Kennedy.
+Portions copyright 2006-2008 Adam Kennedy.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
