@@ -1,12 +1,15 @@
 package TAP::Parser::YAMLish::Writer;
 
 use strict;
+use vars qw($VERSION @ISA);
 
-use vars qw{$VERSION};
+use TAP::Object ();
 
-$VERSION = '3.00';
+@ISA     = 'TAP::Object';
+$VERSION = '3.17_01';
 
 my $ESCAPE_CHAR = qr{ [ \x00-\x1f \" ] }x;
+my $ESCAPE_KEY  = qr{ (?: ^\W ) | $ESCAPE_CHAR }x;
 
 my @UNPRINTABLE = qw(
   z    x01  x02  x03  x04  x05  x06  a
@@ -15,11 +18,7 @@ my @UNPRINTABLE = qw(
   x18  x19  x1a  e    x1c  x1d  x1e  x1f
 );
 
-# Create an empty TAP::Parser::YAMLish::Writer object
-sub new {
-    my $class = shift;
-    bless {}, $class;
-}
+# new() implementation supplied by TAP::Object
 
 sub write {
     my $self = shift;
@@ -71,10 +70,11 @@ sub _put {
 sub _enc_scalar {
     my $self = shift;
     my $val  = shift;
+    my $rule = shift;
 
     return '~' unless defined $val;
 
-    if ( $val =~ /$ESCAPE_CHAR/ ) {
+    if ( $val =~ /$rule/ ) {
         $val =~ s/\\/\\\\/g;
         $val =~ s/"/\\"/g;
         $val =~ s/ ( [\x00-\x1f] ) / '\\' . $UNPRINTABLE[ ord($1) ] /gex;
@@ -103,7 +103,7 @@ sub _write_obj {
                 for my $key ( sort keys %$obj ) {
                     my $value = $obj->{$key};
                     $self->_write_obj(
-                        $pad . $self->_enc_scalar($key) . ':',
+                        $pad . $self->_enc_scalar( $key, $ESCAPE_KEY ) . ':',
                         $value, $indent + 1
                     );
                 }
@@ -127,11 +127,11 @@ sub _write_obj {
             }
         }
         else {
-            die "Don't know how to enocde $ref";
+            die "Don't know how to encode $ref";
         }
     }
     else {
-        $self->_put( $prefix, ' ', $self->_enc_scalar($obj) );
+        $self->_put( $prefix, ' ', $self->_enc_scalar( $obj, $ESCAPE_CHAR ) );
     }
 }
 
@@ -147,7 +147,7 @@ TAP::Parser::YAMLish::Writer - Write YAMLish data
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =head1 SYNOPSIS
 
@@ -243,7 +243,7 @@ L<http://use.perl.org/~Alias/journal/29427>
 
 =head1 COPYRIGHT
 
-Copyright 2007 Andy Armstrong.
+Copyright 2007-2008 Andy Armstrong.
 
 This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
