@@ -7,9 +7,13 @@ use strict;
 use constant IS_WIN32 => ( $^O =~ /^(MS)?Win32$/ );
 use constant IS_VMS => ( $^O eq 'VMS' );
 
-use TAP::Harness            ();
-use TAP::Parser::Aggregator ();
+use TAP::Harness              ();
+use TAP::Parser::Aggregator   ();
+use TAP::Parser::Source::Perl ();
 
+use TAP::Parser::Utils qw( split_shell );
+
+use Config;
 use Exporter;
 
 # TODO: Emulate at least some of these
@@ -19,10 +23,13 @@ use vars qw(
   $Verbose $Switches $Debug
   $verbose $switches $debug
   $Columns
+  $Color
   $Directives
   $Timer
   $Strap
+  $HarnessSubclass
   $has_time_hires
+  $IgnoreExit
 );
 
 # $ML $Last_ML_Print
@@ -38,11 +45,11 @@ Test::Harness - Run Perl standard test scripts with statistics
 
 =head1 VERSION
 
-Version 3.00
+Version 3.17_01
 
 =cut
 
-$VERSION = '3.00';
+$VERSION = '3.17_01';
 
 # Backwards compatibility for exportable variable names.
 *verbose  = *Verbose;
@@ -68,7 +75,9 @@ $Debug   = $ENV{HARNESS_DEBUG}   || 0;
 $Switches = '-w';
 $Columns = $ENV{HARNESS_COLUMNS} || $ENV{COLUMNS} || 80;
 $Columns--;    # Some shells have trouble with a full line of text.
-$Timer = $ENV{HARNESS_TIMER} || 0;
+$Timer      = $ENV{HARNESS_TIMER}       || 0;
+$Color      = $ENV{HARNESS_COLOR}       || 0;
+$IgnoreExit = $ENV{HARNESS_IGNORE_EXIT} || 0;
 
 =head1 SYNOPSIS
 
@@ -89,7 +98,8 @@ pluggable 'Straps' interface that previous versions of L<Test::Harness>
 supported is not reproduced here. Straps is now available as a stand
 alone module: L<Test::Harness::Straps>.
 
-See L<TAP::Parser> for the main documentation for this distribution.
+See L<TAP::Parser>, L<TAP::Harness> for the main documentation for this
+distribution.
 
 =head1 FUNCTIONS
 
@@ -107,6 +117,42 @@ one of the messages in the DIAGNOSTICS section.
 
 =cut
 
+sub _has_taint {
+    my $test = shift;
+    return TAP::Parser::Source::Perl->get_taint(
+        TAP::Parser::Source::Perl->shebang($test) );
+}
+
+sub _aggregate {
+    my ( $harness, $aggregate, @tests ) = @_;
+
+    # Don't propagate to our children
+    local $ENV{HARNESS_OPTIONS};
+
+    _apply_extra_INC($harness);
+    _aggregate_tests( $harness, $aggregate, @tests );
+}
+
+# Make sure the child seens all the extra junk in @INC
+sub _apply_extra_INC {
+    my $harness = shift;
+
+    $harness->callback(
+        parser_args => sub {
+            my ( $args, $test ) = @_;
+            push @{ $args->{switches} }, map {"-I$_"} _filtered_inc();
+        }
+    );
+}
+
+sub _aggregate_tests {
+    my ( $harness, $aggregate, @tests ) = @_;
+    $aggregate->start();
+    $harness->aggregate_tests( $aggregate, @tests );
+    $aggregate->stop();
+
+}
+
 sub runtests {
     my @tests = @_;
 
@@ -116,7 +162,7 @@ sub runtests {
     my $harness   = _new_harness();
     my $aggregate = TAP::Parser::Aggregator->new();
 
-    $harness->aggregate_tests( $aggregate, @tests );
+    _aggregate( $harness, $aggregate, @tests );
 
     $harness->formatter->summary($aggregate);
 
@@ -159,40 +205,60 @@ sub _canon {
 }
 
 sub _new_harness {
+    my $sub_args = shift || {};
 
-    if ( defined( my $env_sw = $ENV{HARNESS_PERL_SWITCHES} ) ) {
-        $Switches .= ' ' . $env_sw if ( length($env_sw) );
-    }
-
-    # This is a bit crufty. The switches have all been joined into a
-    # single string so we have to try and recover them.
     my ( @lib, @switches );
-    for my $opt ( split( /\s+(?=-)/, $Switches ) ) {
+    my @opt = split_shell( $Switches, $ENV{HARNESS_PERL_SWITCHES} );
+    while ( my $opt = shift @opt ) {
         if ( $opt =~ /^ -I (.*) $ /x ) {
-            push @lib, $1;
+            push @lib, length($1) ? $1 : shift @opt;
         }
         else {
             push @switches, $opt;
         }
     }
 
-    push @lib, _filtered_inc();
+    # Do things the old way on VMS...
+    push @lib, _filtered_inc() if IS_VMS;
+
+    # If $Verbose isn't numeric default to 1. This helps core.
+    my $verbosity = ( $Verbose ? ( $Verbose !~ /\d/ ) ? 1 : $Verbose : 0 );
 
     my $args = {
-        timer      => $Timer,
-        directives => $Directives,
-        lib        => \@lib,
-        switches   => \@switches,
-        verbosity  => $Verbose,
+        timer       => $Timer,
+        directives  => $Directives,
+        lib         => \@lib,
+        switches    => \@switches,
+        color       => $Color,
+        verbosity   => $verbosity,
+        ignore_exit => $IgnoreExit,
     };
 
-    return TAP::Harness->new($args);
+    $args->{stdout} = $sub_args->{out}
+      if exists $sub_args->{out};
+
+    if ( defined( my $env_opt = $ENV{HARNESS_OPTIONS} ) ) {
+        for my $opt ( split /:/, $env_opt ) {
+            if ( $opt =~ /^j(\d*)$/ ) {
+                $args->{jobs} = $1 || 9;
+            }
+            elsif ( $opt eq 'c' ) {
+                $args->{color} = 1;
+            }
+            else {
+                die "Unknown HARNESS_OPTIONS item: $opt\n";
+            }
+        }
+    }
+
+    my $class = $ENV{HARNESS_SUBCLASS} || 'TAP::Harness';
+    return TAP::Harness->_construct( $class, $args );
 }
 
 # Get the parts of @INC which are changed from the stock list AND
 # preserve reordering of stock directories.
 sub _filtered_inc {
-    my @inc = grep { !ref } @INC; #28567
+    my @inc = grep { !ref } @INC;    #28567
 
     if (IS_VMS) {
 
@@ -204,7 +270,7 @@ sub _filtered_inc {
     elsif (IS_WIN32) {
 
         # Lose any trailing backslashes in the Win32 paths
-        s/[\\\/+]$// foreach @inc;
+        s/[\\\/]+$// foreach @inc;
     }
 
     my @default_inc = _default_inc();
@@ -234,8 +300,14 @@ sub _filtered_inc {
 
     sub _default_inc {
         return @inc if @inc;
+
+        local $ENV{PERL5LIB};
+        local $ENV{PERLLIB};
+
         my $perl = $ENV{HARNESS_PERL} || $^X;
-        chomp( @inc = `$perl -le "print join qq[\\n], \@INC"` );
+
+        # Avoid using -l for the benefit of Perl 6
+        chomp( @inc = `$perl -e "print join qq[\\n], \@INC, q[]"` );
         return @inc;
     }
 }
@@ -254,9 +326,7 @@ sub _check_sequence {
 sub execute_tests {
     my %args = @_;
 
-    # TODO: Handle out option
-
-    my $harness   = _new_harness();
+    my $harness   = _new_harness( \%args );
     my $aggregate = TAP::Parser::Aggregator->new();
 
     my %tot = (
@@ -289,7 +359,7 @@ sub execute_tests {
         }
     );
 
-    $harness->aggregate_tests( $aggregate, @{ $args{tests} } );
+    _aggregate( $harness, $aggregate, @{ $args{tests} } );
 
     $tot{bench} = $aggregate->elapsed;
     my @tests = $aggregate->descriptions;
@@ -450,12 +520,43 @@ If true, C<Test::Harness> will output the verbose results of running
 its tests.  Setting C<$Test::Harness::verbose> will override this,
 or you can use the C<-v> switch in the F<prove> utility.
 
-If true, C<Test::Harness> will output the verbose results of running
-its tests.  Setting C<$Test::Harness::verbose> will override this,
-or you can use the C<-v> switch in the F<prove> utility.
+=item C<HARNESS_OPTIONS>
+
+Provide additional options to the harness. Currently supported options are:
+
+=over
+
+=item C<< j<n> >>
+
+Run <n> (default 9) parallel jobs.
+
+=item C<< f >>
+
+Use forked parallelism.
 
 =back
 
+Multiple options may be separated by colons:
+
+    HARNESS_OPTIONS=j9:f make test
+
+=item C<HARNESS_SUBCLASS>
+
+Specifies a TAP::Harness subclass to be used in place of TAP::Harness.
+
+=back
+
+=head1 Taint Mode
+
+Normally when a Perl program is run in taint mode the contents of the
+C<PERL5LIB> environment variable do not appear in C<@INC>.
+
+Because C<PERL5LIB> is often used during testing to add build
+directories to C<@INC> C<Test::Harness> (actually
+L<TAP::Parser::Source::Perl>) passes the names of any directories found
+in C<PERL5LIB> as -I switches. The net effect of this is that
+C<PERL5LIB> is honoured even in taint mode.
+
 =head1 SEE ALSO
 
 L<TAP::Harness>
@@ -472,7 +573,8 @@ as I make changes.
 
 Andy Armstrong  C<< <andy@hexten.net> >>
 
-L<Test::Harness> (on which this module is based) has this attribution:
+L<Test::Harness> 2.64 (maintained by Andy Lester and on which this
+module is based) has this attribution:
 
     Either Tim Bunce or Andreas Koenig, we don't know. What we know for
     sure is, that it was inspired by Larry Wall's F<TEST> script that came
@@ -482,7 +584,7 @@ L<Test::Harness> (on which this module is based) has this attribution:
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2007, Andy Armstrong C<< <andy@hexten.net> >>. All rights reserved.
+Copyright (c) 2007-2008, Andy Armstrong C<< <andy@hexten.net> >>. All rights reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
